@@ -204,7 +204,7 @@ write_buf(
 			print_warning(
 			    "obfuscation corrupted block at %s bno 0x%llx/0x%x",
 				bp->b_ops->name,
-				(long long)bp->b_bn, bp->b_bcount);
+				(long long)bp->b_bn, BBTOB(bp->b_length));
 		}
 	}
 
@@ -246,6 +246,7 @@ zero_btree_node(
 	xfs_alloc_key_t		*akp;
 	char			*zp1, *zp2;
 	char			*key_end;
+	struct xfs_ino_geometry	*igeo = M_IGEO(mp);
 
 	nrecs = be16_to_cpu(block->bb_numrecs);
 	if (nrecs < 0)
@@ -265,11 +266,11 @@ zero_btree_node(
 		break;
 	case TYP_INOBT:
 	case TYP_FINOBT:
-		if (nrecs > mp->m_inobt_mxr[1])
+		if (nrecs > igeo->inobt_mxr[1])
 			return;
 
 		ikp = XFS_INOBT_KEY_ADDR(mp, block, 1);
-		ipp = XFS_INOBT_PTR_ADDR(mp, block, 1, mp->m_inobt_mxr[1]);
+		ipp = XFS_INOBT_PTR_ADDR(mp, block, 1, igeo->inobt_mxr[1]);
 		zp1 = (char *)&ikp[nrecs];
 		zp2 = (char *)&ipp[nrecs];
 		key_end = (char *)ipp;
@@ -328,7 +329,7 @@ zero_btree_leaf(
 		break;
 	case TYP_INOBT:
 	case TYP_FINOBT:
-		if (nrecs > mp->m_inobt_mxr[0])
+		if (nrecs > M_IGEO(mp)->inobt_mxr[0])
 			return;
 
 		irp = XFS_INOBT_REC_ADDR(mp, block, 1);
@@ -1268,7 +1269,7 @@ process_sf_dir(
 			namelen = ino_dir_size - ((char *)&sfep->name[0] -
 					 (char *)sfp);
 		} else if ((char *)sfep - (char *)sfp +
-				M_DIROPS(mp)->sf_entsize(sfp, sfep->namelen) >
+				libxfs_dir2_sf_entsize(mp, sfp, sfep->namelen) >
 				ino_dir_size) {
 			if (show_warnings)
 				print_warning("entry length in dir inode %llu "
@@ -1281,11 +1282,11 @@ process_sf_dir(
 
 		if (obfuscate)
 			generate_obfuscated_name(
-					 M_DIROPS(mp)->sf_get_ino(sfp, sfep),
+					 libxfs_dir2_sf_get_ino(mp, sfp, sfep),
 					 namelen, &sfep->name[0]);
 
 		sfep = (xfs_dir2_sf_entry_t *)((char *)sfep +
-				M_DIROPS(mp)->sf_entsize(sfp, namelen));
+				libxfs_dir2_sf_entsize(mp, sfp, namelen));
 	}
 
 	/* zero stale data in rest of space in data fork, if any */
@@ -1362,19 +1363,19 @@ process_sf_symlink(
 
 static void
 process_sf_attr(
-	xfs_dinode_t		*dip)
+	xfs_dinode_t			*dip)
 {
 	/*
 	 * with extended attributes, obfuscate the names and fill the actual
 	 * values with 'v' (to see a valid string length, as opposed to NULLs)
 	 */
 
-	xfs_attr_shortform_t	*asfp;
-	xfs_attr_sf_entry_t	*asfep;
-	int			ino_attr_size;
-	int			i;
+	struct xfs_attr_shortform	*asfp;
+	struct xfs_attr_sf_entry	*asfep;
+	int				ino_attr_size;
+	int				i;
 
-	asfp = (xfs_attr_shortform_t *)XFS_DFORK_APTR(dip);
+	asfp = (struct xfs_attr_shortform *)XFS_DFORK_APTR(dip);
 	if (asfp->hdr.count == 0)
 		return;
 
@@ -1398,7 +1399,7 @@ process_sf_attr(
 						"%llu", (long long)cur_ino);
 			break;
 		} else if ((char *)asfep - (char *)asfp +
-				XFS_ATTR_SF_ENTSIZE(asfep) > ino_attr_size) {
+				xfs_attr_sf_entsize(asfep) > ino_attr_size) {
 			if (show_warnings)
 				print_warning("attr entry length in inode %llu "
 					"overflows space", (long long)cur_ino);
@@ -1412,8 +1413,8 @@ process_sf_attr(
 			       asfep->valuelen);
 		}
 
-		asfep = (xfs_attr_sf_entry_t *)((char *)asfep +
-				XFS_ATTR_SF_ENTSIZE(asfep));
+		asfep = (struct xfs_attr_sf_entry *)((char *)asfep +
+				xfs_attr_sf_entsize(asfep));
 	}
 
 	/* zero stale data in rest of space in attr fork, if any */
@@ -1421,34 +1422,86 @@ process_sf_attr(
 		memset(asfep, 0, XFS_DFORK_ASIZE(dip, mp) - ino_attr_size);
 }
 
+static void
+process_dir_free_block(
+	char				*block)
+{
+	struct xfs_dir2_free		*free;
+	struct xfs_dir3_icfree_hdr	freehdr;
+
+	if (!zero_stale_data)
+		return;
+
+	free = (struct xfs_dir2_free *)block;
+	libxfs_dir2_free_hdr_from_disk(mp, &freehdr, free);
+
+	switch (freehdr.magic) {
+	case XFS_DIR2_FREE_MAGIC:
+	case XFS_DIR3_FREE_MAGIC: {
+		__be16			*bests;
+		char			*high;
+		int			used;
+
+		/* Zero out space from end of bests[] to end of block */
+		bests = freehdr.bests;
+		high = (char *)&bests[freehdr.nvalid];
+		used = high - (char*)free;
+		memset(high, 0, mp->m_dir_geo->blksize - used);
+		iocur_top->need_crc = 1;
+		break;
+	}
+	default:
+		if (show_warnings)
+			print_warning("invalid magic in dir inode %llu "
+				      "free block",
+				      (unsigned long long)cur_ino);
+		break;
+	}
+}
+
 static void
 process_dir_leaf_block(
 	char				*block)
 {
 	struct xfs_dir2_leaf		*leaf;
-	struct xfs_dir3_icleaf_hdr 	leafhdr;
+	struct xfs_dir3_icleaf_hdr	leafhdr;
 
 	if (!zero_stale_data)
 		return;
 
 	/* Yes, this works for dir2 & dir3.  Difference is padding. */
 	leaf = (struct xfs_dir2_leaf *)block;
-	M_DIROPS(mp)->leaf_hdr_from_disk(&leafhdr, leaf);
+	libxfs_dir2_leaf_hdr_from_disk(mp, &leafhdr, leaf);
 
-	/* Zero out space from end of ents[] to bests */
-	if (leafhdr.magic == XFS_DIR2_LEAF1_MAGIC ||
-	    leafhdr.magic == XFS_DIR3_LEAF1_MAGIC) {
+	switch (leafhdr.magic) {
+	case XFS_DIR2_LEAF1_MAGIC:
+	case XFS_DIR3_LEAF1_MAGIC: {
 		struct xfs_dir2_leaf_tail	*ltp;
 		__be16				*lbp;
-		struct xfs_dir2_leaf_entry	*ents;
 		char				*free; /* end of ents */
 
-		ents = M_DIROPS(mp)->leaf_ents_p(leaf);
-		free = (char *)&ents[leafhdr.count];
+		/* Zero out space from end of ents[] to bests */
+		free = (char *)&leafhdr.ents[leafhdr.count];
 		ltp = xfs_dir2_leaf_tail_p(mp->m_dir_geo, leaf);
 		lbp = xfs_dir2_leaf_bests_p(ltp);
 		memset(free, 0, (char *)lbp - free);
 		iocur_top->need_crc = 1;
+		break;
+	}
+	case XFS_DIR2_LEAFN_MAGIC:
+	case XFS_DIR3_LEAFN_MAGIC: {
+		char				*free;
+		int				used;
+
+		/* Zero out space from end of ents[] to end of block */
+		free = (char *)&leafhdr.ents[leafhdr.count];
+		used = free - (char*)leaf;
+		memset(free, 0, mp->m_dir_geo->blksize - used);
+		iocur_top->need_crc = 1;
+		break;
+	}
+	default:
+		break;
 	}
 }
 
@@ -1499,11 +1552,11 @@ process_dir_data_block(
 		if (show_warnings)
 			print_warning(
 		"invalid magic in dir inode %llu block %ld",
-					(long long)cur_ino, (long)offset);
+					(unsigned long long)cur_ino, (long)offset);
 		return;
 	}
 
-	dir_offset = M_DIROPS(mp)->data_entry_offset;
+	dir_offset = mp->m_dir_geo->data_entry_offset;
 	ptr = block + dir_offset;
 	endptr = block + mp->m_dir_geo->blksize;
 
@@ -1549,7 +1602,7 @@ process_dir_data_block(
 		}
 
 		dep = (xfs_dir2_data_entry_t *)ptr;
-		length = M_DIROPS(mp)->data_entsize(dep->namelen);
+		length = libxfs_dir2_data_entsize(mp, dep->namelen);
 
 		if (dir_offset + length > end_of_data ||
 		    ptr + length > endptr) {
@@ -1559,7 +1612,7 @@ process_dir_data_block(
 					(long long)cur_ino);
 			return;
 		}
-		if (be16_to_cpu(*M_DIROPS(mp)->data_entry_tag_p(dep)) !=
+		if (be16_to_cpu(*libxfs_dir2_data_entry_tag_p(mp, dep)) !=
 				dir_offset)
 			return;
 
@@ -1572,7 +1625,7 @@ process_dir_data_block(
 		if (zero_stale_data) {
 			/* 1 byte for ftype; don't bother with conditional */
 			int zlen =
-				(char *)M_DIROPS(mp)->data_entry_tag_p(dep) -
+				(char *)libxfs_dir2_data_entry_tag_p(mp, dep) -
 				(char *)&dep->name[dep->namelen] - 1;
 			if (zlen > 0) {
 				memset(&dep->name[dep->namelen] + 1, 0, zlen);
@@ -1582,11 +1635,34 @@ process_dir_data_block(
 	}
 }
 
-static void
+static int
 process_symlink_block(
-	char			*block)
+	xfs_fileoff_t	o,
+	xfs_fsblock_t	s,
+	xfs_filblks_t	c,
+	typnm_t		btype,
+	xfs_fileoff_t	last)
 {
-	char *link = block;
+	struct bbmap	map;
+	char		*link;
+	int		ret = 0;
+
+	push_cur();
+	map.nmaps = 1;
+	map.b[0].bm_bn = XFS_FSB_TO_DADDR(mp, s);
+	map.b[0].bm_len = XFS_FSB_TO_BB(mp, c);
+	set_cur(&typtab[btype], 0, 0, DB_RING_IGN, &map);
+	if (!iocur_top->data) {
+		xfs_agnumber_t	agno = XFS_FSB_TO_AGNO(mp, s);
+		xfs_agblock_t	agbno = XFS_FSB_TO_AGBNO(mp, s);
+
+		print_warning("cannot read %s block %u/%u (%llu)",
+				typtab[btype].name, agno, agbno, s);
+		if (stop_on_read_error)
+			ret = -1;
+		goto out_pop;
+	}
+	link = iocur_top->data;
 
 	if (xfs_sb_version_hascrc(&(mp)->m_sb))
 		link += sizeof(struct xfs_dsymlink_hdr);
@@ -1604,6 +1680,12 @@ process_symlink_block(
 		if (zlen < mp->m_sb.sb_blocksize)
 			memset(link + linklen, 0, zlen);
 	}
+
+	iocur_top->need_crc = 1;
+	ret = write_buf(iocur_top);
+out_pop:
+	pop_cur();
+	return ret;
 }
 
 #define MAX_REMOTE_VALS		4095
@@ -1795,8 +1877,19 @@ process_single_fsb_objects(
 				struct xfs_da3_icnode_hdr hdr;
 				int used;
 
-				M_DIROPS(mp)->node_hdr_from_disk(&hdr, node);
-				used = M_DIROPS(mp)->node_hdr_size;
+				libxfs_da3_node_hdr_from_disk(mp, &hdr, node);
+				switch (btype) {
+				case TYP_DIR2:
+					used = mp->m_dir_geo->node_hdr_size;
+					break;
+				case TYP_ATTR:
+					used = mp->m_attr_geo->node_hdr_size;
+					break;
+				default:
+					/* unknown type, don't zero anything */
+					used = mp->m_sb.sb_blocksize;
+					break;
+				}
 
 				used += hdr.count
 					* sizeof(struct xfs_da_node_entry);
@@ -1814,8 +1907,7 @@ process_single_fsb_objects(
 		switch (btype) {
 		case TYP_DIR2:
 			if (o >= mp->m_dir_geo->freeblk) {
-				/* TODO, zap any stale data */
-				break;
+				process_dir_free_block(dp);
 			} else if (o >= mp->m_dir_geo->leafblk) {
 				process_dir_leaf_block(dp);
 			} else {
@@ -1824,10 +1916,6 @@ process_single_fsb_objects(
 			}
 			iocur_top->need_crc = 1;
 			break;
-		case TYP_SYMLINK:
-			process_symlink_block(dp);
-			iocur_top->need_crc = 1;
-			break;
 		case TYP_ATTR:
 			process_attr_block(dp, o);
 			iocur_top->need_crc = 1;
@@ -1856,23 +1944,16 @@ static struct bbmap mfsb_map;
 static int mfsb_length;
 
 static int
-process_multi_fsb_objects(
+process_multi_fsb_dir(
 	xfs_fileoff_t	o,
 	xfs_fsblock_t	s,
 	xfs_filblks_t	c,
 	typnm_t		btype,
 	xfs_fileoff_t	last)
 {
+	char		*dp;
 	int		ret = 0;
 
-	switch (btype) {
-	case TYP_DIR2:
-		break;
-	default:
-		print_warning("bad type for multi-fsb object %d", btype);
-		return -EINVAL;
-	}
-
 	while (c > 0) {
 		unsigned int	bm_len;
 
@@ -1903,15 +1984,20 @@ process_multi_fsb_objects(
 
 			}
 
-			if ((!obfuscate && !zero_stale_data) ||
-			     o >= mp->m_dir_geo->leafblk) {
-				ret = write_buf(iocur_top);
-				goto out_pop;
-			}
+			if (!obfuscate && !zero_stale_data)
+				goto write;
 
-			process_dir_data_block(iocur_top->data, o,
-					       last == mp->m_dir_geo->fsbcount);
+			dp = iocur_top->data;
+			if (o >= mp->m_dir_geo->freeblk) {
+				process_dir_free_block(dp);
+			} else if (o >= mp->m_dir_geo->leafblk) {
+				process_dir_leaf_block(dp);
+			} else {
+				process_dir_data_block(dp, o,
+					 last == mp->m_dir_geo->fsbcount);
+			}
 			iocur_top->need_crc = 1;
+write:
 			ret = write_buf(iocur_top);
 out_pop:
 			pop_cur();
@@ -1926,6 +2012,37 @@ out_pop:
 	return ret;
 }
 
+static bool
+is_multi_fsb_object(
+	struct xfs_mount	*mp,
+	typnm_t			btype)
+{
+	if (btype == TYP_DIR2 && mp->m_dir_geo->fsbcount > 1)
+		return true;
+	if (btype == TYP_SYMLINK)
+		return true;
+	return false;
+}
+
+static int
+process_multi_fsb_objects(
+	xfs_fileoff_t	o,
+	xfs_fsblock_t	s,
+	xfs_filblks_t	c,
+	typnm_t		btype,
+	xfs_fileoff_t	last)
+{
+	switch (btype) {
+	case TYP_DIR2:
+		return process_multi_fsb_dir(o, s, c, btype, last);
+	case TYP_SYMLINK:
+		return process_symlink_block(o, s, c, btype, last);
+	default:
+		print_warning("bad type for multi-fsb object %d", btype);
+		return -EINVAL;
+	}
+}
+
 /* inode copy routines */
 static int
 process_bmbt_reclist(
@@ -1941,6 +2058,7 @@ process_bmbt_reclist(
 	xfs_fileoff_t		last;
 	xfs_agnumber_t		agno;
 	xfs_agblock_t		agbno;
+	bool			is_multi_fsb = is_multi_fsb_object(mp, btype);
 	int			error;
 
 	if (btype == TYP_DATA)
@@ -2004,11 +2122,12 @@ process_bmbt_reclist(
 		}
 
 		/* multi-extent blocks require special handling */
-		if (btype != TYP_DIR2 || mp->m_dir_geo->fsbcount == 1) {
-			error = process_single_fsb_objects(o, s, c, btype, last);
-		} else {
-			error = process_multi_fsb_objects(o, s, c, btype, last);
-		}
+		if (is_multi_fsb)
+			error = process_multi_fsb_objects(o, s, c, btype,
+					last);
+		else
+			error = process_single_fsb_objects(o, s, c, btype,
+					last);
 		if (error)
 			return 0;
 	}
@@ -2116,6 +2235,19 @@ process_btinode(
 	}
 
 	pp = XFS_BMDR_PTR_ADDR(dib, 1, maxrecs);
+
+	if (zero_stale_data) {
+		char	*top;
+
+		/* Unused btree key space */
+		top = (char*)XFS_BMDR_KEY_ADDR(dib, nrecs + 1);
+		memset(top, 0, (char*)pp - top);
+
+		/* Unused btree ptr space */
+		top = (char*)&pp[nrecs];
+		memset(top, 0, (char*)dib + XFS_DFORK_SIZE(dip, mp, whichfork) - top);
+	}
+
 	for (i = 0; i < nrecs; i++) {
 		xfs_agnumber_t	ag;
 		xfs_agblock_t	bno;
@@ -2199,6 +2331,26 @@ process_inode_data(
 	return 1;
 }
 
+static int
+process_dev_inode(
+	xfs_dinode_t		*dip)
+{
+	if (XFS_DFORK_NEXTENTS(dip, XFS_DATA_FORK)) {
+		if (show_warnings)
+			print_warning("inode %llu has unexpected extents",
+				      (unsigned long long)cur_ino);
+		return 0;
+	} else {
+		if (zero_stale_data) {
+			unsigned int	size = sizeof(xfs_dev_t);
+
+			memset(XFS_DFORK_DPTR(dip) + size, 0,
+					XFS_DFORK_DSIZE(dip, mp) - size);
+		}
+		return 1;
+	}
+}
+
 /*
  * when we process the inode, we may change the data in the data and/or
  * attribute fork if they are in short form and we are obfuscating names.
@@ -2230,8 +2382,7 @@ process_inode(
 	if (free_inode) {
 		if (zero_stale_data) {
 			/* Zero all of the inode literal area */
-			memset(XFS_DFORK_DPTR(dip), 0,
-			       XFS_LITINO(mp, dip->di_version));
+			memset(XFS_DFORK_DPTR(dip), 0, XFS_LITINO(mp));
 		}
 		goto done;
 	}
@@ -2251,13 +2402,20 @@ process_inode(
 		case S_IFREG:
 			success = process_inode_data(dip, TYP_DATA);
 			break;
-		default: ;
+		case S_IFIFO:
+		case S_IFCHR:
+		case S_IFBLK:
+		case S_IFSOCK:
+			success = process_dev_inode(dip);
+			need_new_crc = 1;
+			break;
+		default:
+			break;
 	}
 	nametable_clear();
 
 	/* copy extended attributes if they exist and forkoff is valid */
-	if (success &&
-	    XFS_DFORK_DSIZE(dip, mp) < XFS_LITINO(mp, dip->di_version)) {
+	if (success && XFS_DFORK_DSIZE(dip, mp) < XFS_LITINO(mp)) {
 		attr_data.remote_val_count = 0;
 		switch (dip->di_aformat) {
 			case XFS_DINODE_FMT_LOCAL:
@@ -2303,10 +2461,11 @@ copy_inode_chunk(
 	int			blks_per_buf;
 	int			inodes_per_buf;
 	int			ioff;
+	struct xfs_ino_geometry *igeo = M_IGEO(mp);
 
 	agino = be32_to_cpu(rp->ir_startino);
 	agbno = XFS_AGINO_TO_AGBNO(mp, agino);
-	end_agbno = agbno + mp->m_ialloc_blks;
+	end_agbno = agbno + igeo->ialloc_blks;
 	off = XFS_INO_TO_OFFSET(mp, agino);
 
 	/*
@@ -2318,10 +2477,10 @@ copy_inode_chunk(
 	 * we've been passed (large block sizes can hold multiple inode chunks).
 	 */
 	if (xfs_sb_version_hassparseinodes(&mp->m_sb))
-		blks_per_buf = xfs_icluster_size_fsb(mp);
+		blks_per_buf = igeo->blocks_per_cluster;
 	else
-		blks_per_buf = mp->m_ialloc_blks;
-	inodes_per_buf = min(blks_per_buf << mp->m_sb.sb_inopblog,
+		blks_per_buf = igeo->ialloc_blks;
+	inodes_per_buf = min(XFS_FSB_TO_INO(mp, blks_per_buf),
 			     XFS_INODES_PER_CHUNK);
 
 	/*
@@ -2420,16 +2579,17 @@ scanfunc_ino(
 	int			i;
 	int			numrecs;
 	int			finobt = *(int *) arg;
+	struct xfs_ino_geometry	*igeo = M_IGEO(mp);
 
 	numrecs = be16_to_cpu(block->bb_numrecs);
 
 	if (level == 0) {
-		if (numrecs > mp->m_inobt_mxr[0]) {
+		if (numrecs > igeo->inobt_mxr[0]) {
 			if (show_warnings)
 				print_warning("invalid numrecs %d in %s "
 					"block %u/%u", numrecs,
 					typtab[btype].name, agno, agbno);
-			numrecs = mp->m_inobt_mxr[0];
+			numrecs = igeo->inobt_mxr[0];
 		}
 
 		/*
@@ -2447,14 +2607,14 @@ scanfunc_ino(
 		return 1;
 	}
 
-	if (numrecs > mp->m_inobt_mxr[1]) {
+	if (numrecs > igeo->inobt_mxr[1]) {
 		if (show_warnings)
 			print_warning("invalid numrecs %d in %s block %u/%u",
 				numrecs, typtab[btype].name, agno, agbno);
-		numrecs = mp->m_inobt_mxr[1];
+		numrecs = igeo->inobt_mxr[1];
 	}
 
-	pp = XFS_INOBT_PTR_ADDR(mp, block, 1, mp->m_inobt_mxr[1]);
+	pp = XFS_INOBT_PTR_ADDR(mp, block, 1, igeo->inobt_mxr[1]);
 	for (i = 0; i < numrecs; i++) {
 		if (!valid_bno(agno, be32_to_cpu(pp[i]))) {
 			if (show_warnings)
@@ -2505,7 +2665,7 @@ copy_inodes(
 		levels = be32_to_cpu(agi->agi_free_level);
 
 		finobt = 1;
-		if (!scan_btree(agno, root, levels, TYP_INOBT, &finobt,
+		if (!scan_btree(agno, root, levels, TYP_FINOBT, &finobt,
 				scanfunc_ino))
 			return 0;
 	}
@@ -2588,7 +2748,7 @@ scan_ag(
 			int i;
 			 __be32  *agfl_bno;
 
-			agfl_bno = XFS_BUF_TO_AGFL_BNO(mp, iocur_top->bp);
+			agfl_bno = xfs_buf_to_agfl_bno(iocur_top->bp);
 			i = be32_to_cpu(agf->agf_fllast);
 
 			for (;;) {
