@@ -81,7 +81,6 @@ char * gettmpname(char *fname);
 char * getparent(char *fname);
 int fsrprintf(const char *fmt, ...);
 int read_fd_bmap(int, struct xfs_bstat *, int *);
-int cmp(const void *, const void *);
 static void tmp_init(char *mnt);
 static char * tmp_next(char *mnt);
 static void tmp_close(char *mnt);
@@ -577,6 +576,23 @@ fsrall_cleanup(int timeout)
 	}
 }
 
+/*
+ * To compare bstat structs for qsort.
+ */
+static int
+cmp(const void *s1, const void *s2)
+{
+	const struct xfs_bulkstat	*bs1 = s1;
+	const struct xfs_bulkstat	*bs2 = s2;
+
+	ASSERT((bs1->bs_version == XFS_BULKSTAT_VERSION_V1 &&
+		bs2->bs_version == XFS_BULKSTAT_VERSION_V1) ||
+		(bs1->bs_version == XFS_BULKSTAT_VERSION_V5 &&
+		bs2->bs_version == XFS_BULKSTAT_VERSION_V5));
+
+	return (bs2->bs_extents - bs1->bs_extents);
+}
+
 /*
  * fsrfs -- reorganize a file system
  */
@@ -696,17 +712,6 @@ out0:
 	return 0;
 }
 
-/*
- * To compare bstat structs for qsort.
- */
-int
-cmp(const void *s1, const void *s2)
-{
-	return( ((struct xfs_bstat *)s2)->bs_extents -
-	        ((struct xfs_bstat *)s1)->bs_extents);
-
-}
-
 /*
  * reorganize by directory hierarchy.
  * Stay in dev (a restriction based on structure of this program -- either
