@@ -208,7 +208,7 @@ rsvfile(
 
 	libxfs_trans_ichgtime(tp, ip, XFS_ICHGTIME_MOD | XFS_ICHGTIME_CHG);
 
-	ip->i_d.di_flags |= XFS_DIFLAG_PREALLOC;
+	ip->i_diflags |= XFS_DIFLAG_PREALLOC;
 
 	libxfs_trans_log_inode(tp, ip, XFS_ILOG_CORE);
 	error = -libxfs_trans_commit(tp);
@@ -281,7 +281,7 @@ newfile(
 			libxfs_buf_relse(bp);
 		}
 	}
-	ip->i_d.di_size = len;
+	ip->i_disk_size = len;
 	return flags;
 }
 
@@ -651,8 +651,8 @@ rtinit(
 	 * commit the transaction in which rbmip was allocated.
 	 */
 	mp->m_sb.sb_rbmino = rbmip->i_ino;
-	rbmip->i_d.di_size = mp->m_sb.sb_rbmblocks * mp->m_sb.sb_blocksize;
-	rbmip->i_d.di_flags = XFS_DIFLAG_NEWRTBM;
+	rbmip->i_disk_size = mp->m_sb.sb_rbmblocks * mp->m_sb.sb_blocksize;
+	rbmip->i_diflags = XFS_DIFLAG_NEWRTBM;
 	*(uint64_t *)&VFS_I(rbmip)->i_atime = 0;
 	libxfs_trans_log_inode(tp, rbmip, XFS_ILOG_CORE);
 	libxfs_log_sb(tp);
@@ -663,7 +663,7 @@ rtinit(
 		fail(_("Realtime summary inode allocation failed"), error);
 	}
 	mp->m_sb.sb_rsumino = rsumip->i_ino;
-	rsumip->i_d.di_size = mp->m_rsumsize;
+	rsumip->i_disk_size = mp->m_rsumsize;
 	libxfs_trans_log_inode(tp, rsumip, XFS_ILOG_CORE);
 	libxfs_log_sb(tp);
 	error = -libxfs_trans_commit(tp);
