@@ -237,6 +237,21 @@ dir_hash_add(
 	return !dup;
 }
 
+/* Mark an existing directory hashtable entry as junk. */
+static void
+dir_hash_junkit(
+	struct dir_hash_tab	*hashtab,
+	xfs_dir2_dataptr_t	addr)
+{
+	struct dir_hash_ent	*p;
+
+	p = radix_tree_lookup(&hashtab->byaddr, addr);
+	assert(p != NULL);
+
+	p->junkit = 1;
+	p->namebuf[0] = '/';
+}
+
 static int
 dir_hash_check(
 	struct dir_hash_tab	*hashtab,
@@ -413,11 +428,10 @@ bmap_next_offset(
 		return EIO;
 	}
 
-	if (!(ip->i_df.if_flags & XFS_IFEXTENTS)) {
-		error = -libxfs_iread_extents(NULL, ip, XFS_DATA_FORK);
-		if (error)
-			return error;
-	}
+        /* Read extent map. */
+	error = -libxfs_iread_extents(NULL, ip, XFS_DATA_FORK);
+	if (error)
+		return error;
 
 	bno = *bnop + 1;
 	if (!libxfs_iext_lookup_extent(ip, &ip->i_df, bno, &icur, &got))
@@ -437,6 +451,22 @@ res_failed(
 		do_error(_("xfs_trans_reserve returned %d\n"), err);
 }
 
+static inline void
+reset_inode_fields(struct xfs_inode *ip)
+{
+	ip->i_projid = 0;
+	ip->i_disk_size = 0;
+	ip->i_nblocks = 0;
+	ip->i_extsize = 0;
+	ip->i_cowextsize = 0;
+	ip->i_flushiter = 0;
+	ip->i_forkoff = 0;
+	ip->i_diflags = 0;
+	ip->i_diflags2 = 0;
+	ip->i_crtime.tv_sec = 0;
+	ip->i_crtime.tv_nsec = 0;
+}
+
 static void
 mk_rbmino(xfs_mount_t *mp)
 {
@@ -465,7 +495,7 @@ mk_rbmino(xfs_mount_t *mp)
 			error);
 	}
 
-	memset(&ip->i_d, 0, sizeof(ip->i_d));
+	reset_inode_fields(ip);
 
 	VFS_I(ip)->i_mode = S_IFREG;
 	ip->i_df.if_format = XFS_DINODE_FMT_EXTENTS;
@@ -477,7 +507,7 @@ mk_rbmino(xfs_mount_t *mp)
 	times = XFS_ICHGTIME_CHG | XFS_ICHGTIME_MOD;
 	if (xfs_sb_version_has_v3inode(&mp->m_sb)) {
 		VFS_I(ip)->i_version = 1;
-		ip->i_d.di_flags2 = 0;
+		ip->i_diflags2 = 0;
 		times |= XFS_ICHGTIME_CREATE;
 	}
 	libxfs_trans_ichgtime(tp, ip, times);
@@ -485,11 +515,10 @@ mk_rbmino(xfs_mount_t *mp)
 	/*
 	 * now the ifork
 	 */
-	ip->i_df.if_flags = XFS_IFEXTENTS;
 	ip->i_df.if_bytes = 0;
 	ip->i_df.if_u1.if_root = NULL;
 
-	ip->i_d.di_size = mp->m_sb.sb_rbmblocks * mp->m_sb.sb_blocksize;
+	ip->i_disk_size = mp->m_sb.sb_rbmblocks * mp->m_sb.sb_blocksize;
 
 	/*
 	 * commit changes
@@ -706,7 +735,7 @@ mk_rsumino(xfs_mount_t *mp)
 			error);
 	}
 
-	memset(&ip->i_d, 0, sizeof(ip->i_d));
+	reset_inode_fields(ip);
 
 	VFS_I(ip)->i_mode = S_IFREG;
 	ip->i_df.if_format = XFS_DINODE_FMT_EXTENTS;
@@ -718,7 +747,7 @@ mk_rsumino(xfs_mount_t *mp)
 	times = XFS_ICHGTIME_CHG | XFS_ICHGTIME_MOD;
 	if (xfs_sb_version_has_v3inode(&mp->m_sb)) {
 		VFS_I(ip)->i_version = 1;
-		ip->i_d.di_flags2 = 0;
+		ip->i_diflags2 = 0;
 		times |= XFS_ICHGTIME_CREATE;
 	}
 	libxfs_trans_ichgtime(tp, ip, times);
@@ -726,11 +755,10 @@ mk_rsumino(xfs_mount_t *mp)
 	/*
 	 * now the ifork
 	 */
-	ip->i_df.if_flags = XFS_IFEXTENTS;
 	ip->i_df.if_bytes = 0;
 	ip->i_df.if_u1.if_root = NULL;
 
-	ip->i_d.di_size = mp->m_rsumsize;
+	ip->i_disk_size = mp->m_rsumsize;
 
 	/*
 	 * commit changes
@@ -806,7 +834,7 @@ mk_root_dir(xfs_mount_t *mp)
 	/*
 	 * take care of the core -- initialization from xfs_ialloc()
 	 */
-	memset(&ip->i_d, 0, sizeof(ip->i_d));
+	reset_inode_fields(ip);
 
 	VFS_I(ip)->i_mode = mode|S_IFDIR;
 	ip->i_df.if_format = XFS_DINODE_FMT_EXTENTS;
@@ -818,7 +846,7 @@ mk_root_dir(xfs_mount_t *mp)
 	times = XFS_ICHGTIME_CHG | XFS_ICHGTIME_MOD;
 	if (xfs_sb_version_has_v3inode(&mp->m_sb)) {
 		VFS_I(ip)->i_version = 1;
-		ip->i_d.di_flags2 = 0;
+		ip->i_diflags2 = 0;
 		times |= XFS_ICHGTIME_CREATE;
 	}
 	libxfs_trans_ichgtime(tp, ip, times);
@@ -828,7 +856,6 @@ mk_root_dir(xfs_mount_t *mp)
 	/*
 	 * now the ifork
 	 */
-	ip->i_df.if_flags = XFS_IFEXTENTS;
 	ip->i_df.if_bytes = 0;
 	ip->i_df.if_u1.if_root = NULL;
 
@@ -1717,6 +1744,7 @@ longform_dir2_entry_check_data(
 				if (entry_junked(
 	_("entry \"%s\" (ino %" PRIu64 ") in dir %" PRIu64 " is not in the the first block"), fname,
 						inum, ip->i_ino)) {
+					dir_hash_junkit(hashtab, addr);
 					dep->name[0] = '/';
 					libxfs_dir2_data_log_entry(&da, bp, dep);
 				}
@@ -1744,6 +1772,7 @@ longform_dir2_entry_check_data(
 				if (entry_junked(
 	_("entry \"%s\" in dir %" PRIu64 " is not the first entry"),
 						fname, inum, ip->i_ino)) {
+					dir_hash_junkit(hashtab, addr);
 					dep->name[0] = '/';
 					libxfs_dir2_data_log_entry(&da, bp, dep);
 				}
@@ -1832,6 +1861,7 @@ _("entry \"%s\" in dir inode %" PRIu64 " inconsistent with .. value (%" PRIu64 "
 				orphanage_ino = 0;
 			nbad++;
 			if (!no_modify)  {
+				dir_hash_junkit(hashtab, addr);
 				dep->name[0] = '/';
 				libxfs_dir2_data_log_entry(&da, bp, dep);
 				if (verbose)
@@ -2207,14 +2237,14 @@ longform_dir2_entry_check(
 	struct xfs_da_args	args;
 
 	*need_dot = 1;
-	freetab = malloc(FREETAB_SIZE(ip->i_d.di_size / mp->m_dir_geo->blksize));
+	freetab = malloc(FREETAB_SIZE(ip->i_disk_size / mp->m_dir_geo->blksize));
 	if (!freetab) {
 		do_error(_("malloc failed in %s (%" PRId64 " bytes)\n"),
 			__func__,
-			FREETAB_SIZE(ip->i_d.di_size / mp->m_dir_geo->blksize));
+			FREETAB_SIZE(ip->i_disk_size / mp->m_dir_geo->blksize));
 		exit(1);
 	}
-	freetab->naents = ip->i_d.di_size / mp->m_dir_geo->blksize;
+	freetab->naents = ip->i_disk_size / mp->m_dir_geo->blksize;
 	freetab->nents = 0;
 	for (i = 0; i < freetab->naents; i++) {
 		freetab->ents[i].v = NULLDATAOFF;
@@ -2427,7 +2457,7 @@ shortform_dir2_entry_check(
 	bytes_deleted = 0;
 
 	max_size = ifp->if_bytes;
-	ASSERT(ip->i_d.di_size <= ifp->if_bytes);
+	ASSERT(ip->i_disk_size <= ifp->if_bytes);
 
 	/*
 	 * if just rebuild a directory due to a "..", update and return
@@ -2491,7 +2521,7 @@ shortform_dir2_entry_check(
 			bad_sfnamelen = 1;
 
 			if (i == sfp->count - 1)  {
-				namelen = ip->i_d.di_size -
+				namelen = ip->i_disk_size -
 					((intptr_t) &sfep->name[0] -
 					 (intptr_t) sfp);
 			} else  {
@@ -2503,11 +2533,11 @@ shortform_dir2_entry_check(
 			}
 		} else if (no_modify && (intptr_t) sfep - (intptr_t) sfp +
 				+ libxfs_dir2_sf_entsize(mp, sfp, sfep->namelen)
-				> ip->i_d.di_size)  {
+				> ip->i_disk_size)  {
 			bad_sfnamelen = 1;
 
 			if (i == sfp->count - 1)  {
-				namelen = ip->i_d.di_size -
+				namelen = ip->i_disk_size -
 					((intptr_t) &sfep->name[0] -
 					 (intptr_t) sfp);
 			} else  {
@@ -2725,17 +2755,17 @@ _("entry \"%s\" (ino %" PRIu64 ") in dir %" PRIu64 " is a duplicate name"),
 	if (*ino_dirty && bytes_deleted > 0)  {
 		ASSERT(!no_modify);
 		libxfs_idata_realloc(ip, -bytes_deleted, XFS_DATA_FORK);
-		ip->i_d.di_size -= bytes_deleted;
+		ip->i_disk_size -= bytes_deleted;
 	}
 
-	if (ip->i_d.di_size != ip->i_df.if_bytes)  {
+	if (ip->i_disk_size != ip->i_df.if_bytes)  {
 		ASSERT(ip->i_df.if_bytes == (xfs_fsize_t)
 				((intptr_t) next_sfep - (intptr_t) sfp));
-		ip->i_d.di_size = (xfs_fsize_t)
+		ip->i_disk_size = (xfs_fsize_t)
 				((intptr_t) next_sfep - (intptr_t) sfp);
 		do_warn(
 	_("setting size to %" PRId64 " bytes to reflect junked entries\n"),
-			ip->i_d.di_size);
+			ip->i_disk_size);
 		*ino_dirty = 1;
 	}
 }
@@ -2809,7 +2839,7 @@ process_dir_inode(
 
 	add_inode_refchecked(irec, ino_offset);
 
-	hashtab = dir_hash_init(ip->i_d.di_size);
+	hashtab = dir_hash_init(ip->i_disk_size);
 
 	/*
 	 * look for bogus entries
